
package org.mule.modules.workday.staffing.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.workday.staffing.config.GetApplicantsMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class GetApplicantsDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public GetApplicantsDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(GetApplicantsMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element employeeReferenceListElement = null;
        employeeReferenceListElement = DomUtils.getChildElementByTagName(element, "employee-reference");
        List<Element> employeeReferenceListChilds = null;
        if (employeeReferenceListElement!= null) {
            String employeeReferenceRef = employeeReferenceListElement.getAttribute("ref");
            if ((employeeReferenceRef!= null)&&(!StringUtils.isBlank(employeeReferenceRef))) {
                if ((!employeeReferenceRef.startsWith(patternInfo.getPrefix()))&&(!employeeReferenceRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("employeeReference", new RuntimeBeanReference(employeeReferenceRef));
                } else {
                    builder.addPropertyValue("employeeReference", employeeReferenceRef);
                }
            } else {
                ManagedMap employeeReference = new ManagedMap();
                employeeReferenceListChilds = DomUtils.getChildElementsByTagName(employeeReferenceListElement, "employee-reference");
                if (employeeReferenceListChilds!= null) {
                    if (employeeReferenceListChilds.size() == 0) {
                        employeeReferenceListChilds = DomUtils.getChildElements(employeeReferenceListElement);
                    }
                    for (Element employeeReferenceChild: employeeReferenceListChilds) {
                        String employeeReferenceValueRef = employeeReferenceChild.getAttribute("value-ref");
                        String employeeReferenceKeyRef = employeeReferenceChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((employeeReferenceValueRef!= null)&&(!StringUtils.isBlank(employeeReferenceValueRef))) {
                            valueObject = new RuntimeBeanReference(employeeReferenceValueRef);
                        } else {
                            valueObject = employeeReferenceChild.getTextContent();
                        }
                        if ((employeeReferenceKeyRef!= null)&&(!StringUtils.isBlank(employeeReferenceKeyRef))) {
                            keyObject = new RuntimeBeanReference(employeeReferenceKeyRef);
                        } else {
                            keyObject = employeeReferenceChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = employeeReferenceChild.getTagName();
                        }
                        employeeReference.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("employeeReference", employeeReference);
            }
        }
        Element referencesListElement = null;
        referencesListElement = DomUtils.getChildElementByTagName(element, "references");
        List<Element> referencesListChilds = null;
        if (referencesListElement!= null) {
            String referencesRef = referencesListElement.getAttribute("ref");
            if ((referencesRef!= null)&&(!StringUtils.isBlank(referencesRef))) {
                if ((!referencesRef.startsWith(patternInfo.getPrefix()))&&(!referencesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("references", new RuntimeBeanReference(referencesRef));
                } else {
                    builder.addPropertyValue("references", referencesRef);
                }
            } else {
                ManagedList references = new ManagedList();
                referencesListChilds = DomUtils.getChildElementsByTagName(referencesListElement, "reference");
                if (referencesListChilds!= null) {
                    for (Element referencesChild: referencesListChilds) {
                        String valueRef = referencesChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            references.add(new RuntimeBeanReference(valueRef));
                        } else {
                            List<Element> innerreferenceListChilds = null;
                            if (referencesChild!= null) {
                                String innerreferenceRef = referencesChild.getAttribute("ref");
                                if ((innerreferenceRef!= null)&&(!StringUtils.isBlank(innerreferenceRef))) {
                                    if ((!innerreferenceRef.startsWith(patternInfo.getPrefix()))&&(!innerreferenceRef.endsWith(patternInfo.getSuffix()))) {
                                        builder.addPropertyValue("inner-reference", new RuntimeBeanReference(innerreferenceRef));
                                    } else {
                                        builder.addPropertyValue("inner-reference", innerreferenceRef);
                                    }
                                } else {
                                    ManagedMap innerreference = new ManagedMap();
                                    innerreferenceListChilds = DomUtils.getChildElementsByTagName(referencesChild, "inner-reference");
                                    if (innerreferenceListChilds!= null) {
                                        if (innerreferenceListChilds.size() == 0) {
                                            innerreferenceListChilds = DomUtils.getChildElements(referencesChild);
                                        }
                                        for (Element innerreferenceChild: innerreferenceListChilds) {
                                            String innerreferenceValueRef = innerreferenceChild.getAttribute("value-ref");
                                            String innerreferenceKeyRef = innerreferenceChild.getAttribute("key-ref");
                                            Object valueObject = null;
                                            Object keyObject = null;
                                            if ((innerreferenceValueRef!= null)&&(!StringUtils.isBlank(innerreferenceValueRef))) {
                                                valueObject = new RuntimeBeanReference(innerreferenceValueRef);
                                            } else {
                                                valueObject = innerreferenceChild.getTextContent();
                                            }
                                            if ((innerreferenceKeyRef!= null)&&(!StringUtils.isBlank(innerreferenceKeyRef))) {
                                                keyObject = new RuntimeBeanReference(innerreferenceKeyRef);
                                            } else {
                                                keyObject = innerreferenceChild.getAttribute("key");
                                            }
                                            if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                                                keyObject = innerreferenceChild.getTagName();
                                            }
                                            innerreference.put(keyObject, valueObject);
                                        }
                                    }
                                    references.add(innerreference);
                                }
                            }
                        }
                    }
                }
                builder.addPropertyValue("references", references);
            }
        }
        Element filterListElement = null;
        filterListElement = DomUtils.getChildElementByTagName(element, "filter");
        List<Element> filterListChilds = null;
        if (filterListElement!= null) {
            String filterRef = filterListElement.getAttribute("ref");
            if ((filterRef!= null)&&(!StringUtils.isBlank(filterRef))) {
                if ((!filterRef.startsWith(patternInfo.getPrefix()))&&(!filterRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("filter", new RuntimeBeanReference(filterRef));
                } else {
                    builder.addPropertyValue("filter", filterRef);
                }
            } else {
                ManagedMap filter = new ManagedMap();
                filterListChilds = DomUtils.getChildElementsByTagName(filterListElement, "filter");
                if (filterListChilds!= null) {
                    if (filterListChilds.size() == 0) {
                        filterListChilds = DomUtils.getChildElements(filterListElement);
                    }
                    for (Element filterChild: filterListChilds) {
                        String filterValueRef = filterChild.getAttribute("value-ref");
                        String filterKeyRef = filterChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((filterValueRef!= null)&&(!StringUtils.isBlank(filterValueRef))) {
                            valueObject = new RuntimeBeanReference(filterValueRef);
                        } else {
                            valueObject = filterChild.getTextContent();
                        }
                        if ((filterKeyRef!= null)&&(!StringUtils.isBlank(filterKeyRef))) {
                            keyObject = new RuntimeBeanReference(filterKeyRef);
                        } else {
                            keyObject = filterChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = filterChild.getTagName();
                        }
                        filter.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("filter", filter);
            }
        }
        Element groupListElement = null;
        groupListElement = DomUtils.getChildElementByTagName(element, "group");
        List<Element> groupListChilds = null;
        if (groupListElement!= null) {
            String groupRef = groupListElement.getAttribute("ref");
            if ((groupRef!= null)&&(!StringUtils.isBlank(groupRef))) {
                if ((!groupRef.startsWith(patternInfo.getPrefix()))&&(!groupRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("group", new RuntimeBeanReference(groupRef));
                } else {
                    builder.addPropertyValue("group", groupRef);
                }
            } else {
                ManagedMap group = new ManagedMap();
                groupListChilds = DomUtils.getChildElementsByTagName(groupListElement, "group");
                if (groupListChilds!= null) {
                    if (groupListChilds.size() == 0) {
                        groupListChilds = DomUtils.getChildElements(groupListElement);
                    }
                    for (Element groupChild: groupListChilds) {
                        String groupValueRef = groupChild.getAttribute("value-ref");
                        String groupKeyRef = groupChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((groupValueRef!= null)&&(!StringUtils.isBlank(groupValueRef))) {
                            valueObject = new RuntimeBeanReference(groupValueRef);
                        } else {
                            valueObject = groupChild.getTextContent();
                        }
                        if ((groupKeyRef!= null)&&(!StringUtils.isBlank(groupKeyRef))) {
                            keyObject = new RuntimeBeanReference(groupKeyRef);
                        } else {
                            keyObject = groupChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = groupChild.getTagName();
                        }
                        group.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("group", group);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
